# Internet Services Architectures

## Microservices - Load Balancer

Michał Wójcik

---

### Load balancer

Why use **load balancer**:

* services can be overwhelmed with requests,
* there can be multiple instances of the same service,
* there can be service discovery,
* client selecting always first instance from list is not a sulution.

---

### Web server example

Web server example:

* load balancer in gateway proxy,
* all client requests go through gateway,
* multiple instance of web server serving the same content,
* gateway forwards client request to selected server,
* gateway returns response to client,
* gateway with load balancer is the only public endpoint to access web servers.

---

### Routing algorithms

**Routing algorithms**:

* **round-robin** - sends subsequent requests in a cyclic loop to all servers,
* **least connections** - load balancer keeps track of the number of active connections to each server, sends new
  requests to servers with the fewest active connections,
* **weighted** round-robin/least connections - different weight for servers (e.g.: different computational power),
* **random** - simple but does not guarantee even,
* **chained failover** - predefined chain of servers, all requests are sent to a server until it can't serve mor
  requests,
* **data ranges** - different servers are responsible for processing different data ranges (requires domain context).

---

### Load balancer types

**Load balancer**:

* **Layer 4** - operate at the transport layer of the OSI model and are responsible for distributing incoming traffic
  based on the source and destination IP addresses and port numbers,
* **Layer 7** - operate at the application layer of the OSI model. They are responsible for distributing incoming
  traffic based on the content of the request.

---

### Load balancer - client side

Local (client side) load balancer is integrated with Spring Cloud discovery service:

```java
@Repository
public class RestRepository {

    @Autowired
    private LoadBalancerClient loadBalancerClient;

    @Autowired
    private RestTemplate restTemplate;

    public void delete() {
        URI uri = loadBalancerClient.choose("library")
                .getUri();
        restTemplate.delete(uri + "/" + id);

    }

}
```

---

### Load balancer - gateway

Load balancer can be also used automatically in Gateway:

```jshelllanguage
@Bean
public RouteLocator customRouteLocator(RouteLocatorBuilder builder) {
    return builder
            .routes()
            .route("library",r->r
                    .host("localhost:8080")
                    .and()
                    .path("/api/books","/api/books/**")
                    .uri("lb://library"))
            .build();
}
```

---

### Helpful resources

Helpful resources:

* <https://www.baeldung.com/cs/load-balancer>
* <https://www.baeldung.com/spring-cloud-load-balancer>
* <https://www.baeldung.com/spring-cloud-gateway>
