# Internet Services Architectures

## Microservices - configuration

Michał Wójcik

---

### Standard configuration

**Standard configuration**:

* default configuration stored in `application.properties` (for Spring Boot),
* default configuration can be overwritten with environment variables,
* environment variables can be set in containerization environment,
* changing shared configuration requires modification in every module.

---

### Centralized configuration

**Centralized configuration**:

* define configuration for each module in central config server,
* each module connects to config server on start,
* modules do not need to know discovery service address, database address, messaging system address etc.,
* central configuration can be versioned,
* central configuration can be updated during runtime (requires notification system).

---

### Centralized configuration - dependencies

Server dependency:

```xml
<dependency>
    <groupId>org.springframework.cloud</groupId>
    <artifactId>spring-cloud-config-server</artifactId>
</dependency>
```

Client dependency:

```xml
<dependency>
    <groupId>org.springframework.cloud</groupId>
    <artifactId>spring-cloud-starter-config</artifactId>
</dependency>
```

---

### Centralized configuration - server

Server configuration:

```java
@SpringBootApplication
@EnableConfigServer
public class ConfigApplication {

    public static void main(String[] args) {
        SpringApplication.run(ConfigApplication.class, args);
    }

}
```

---

### Centralized configuration - source

By default, configuration is stored in git.

```properties
spring.cloud.config.server.git.uri=ssh://localhost/config-repo
spring.cloud.config.server.git.clone-on-start=true
spring.security.user.name=root
spring.security.user.password=s3cr3t
```

It can be configured to use local (or classpath) files.

```properties
spring.profiles.active=native
spring.cloud.config.server.native.search-locations=classpath:/configuration
```

Defining shared configuration in `/configuration/application.properties` or defining particular service configuration
in `/configuration/library.properties`.

---

### Centralized configuration - client

Configuring client in `bootstrap.properties` (in newer versions in `application.properties`):

```properties
spring.cloud.config.uri=http://localhost:8084
spring.cloud.config.fail-fast=true
```

```properties
spring.config.import=optional:configserver:http://localhost:8084
```

Requires service (application) name in `application.properties:

```properties
spring.application.name=library
```

---

### Helpful resources

Helpful resources:

* <https://www.baeldung.com/spring-cloud-configuration>
* <https://www.baeldung.com/spring-cloud-config-remote-properties-override>
* <https://www.baeldung.com/spring-cloud-config-without-git>
* <https://www.baeldung.com/spring-cloud-bootstrap-properties>
